{% block chartjs_scripts %}
<script src="{{ url_for('bussdcc_framework_chartjs.static', filename='chart.4.5.1.umd.min.js') }}"></script>
<script src="{{ url_for('bussdcc_framework_chartjs.static', filename='chartjs-adapter-date-fns.3.0.0.bundle.min.js') }}"></script>
{% endblock %}
